@@ -9,7 +9,7 @@ use Try::Tiny;
 use Plack::App::WebSocket::Connection;
 use Scalar::Util qw(blessed);
 
-our $VERSION = "0.06";
+our $VERSION = "0.07";
 
 my $ERROR_ENV = "plack.app.websocket.error";
 
@@ -72,10 +72,11 @@ sub call {
         my $responder = shift;
         $cv_conn->cb(sub {
             my ($cv_conn) = @_;
-            my ($conn, $error) = try {
-                (scalar($cv_conn->recv), undef);
+            my ($conn, @handshake_results, $error);
+            try {
+                ($conn, @handshake_results) = $cv_conn->recv;
             }catch {
-                (undef, $_[0]);
+                $error = $_[0];
             };
             if(!$conn) {
                 $env->{$ERROR_ENV} = "invalid request";
@@ -83,7 +84,7 @@ sub call {
                 _respond_via($responder, $self->{on_error}->($env));
                 return;
             }
-            $self->{on_establish}->(Plack::App::WebSocket::Connection->new($conn, $responder), $env);
+            $self->{on_establish}->(Plack::App::WebSocket::Connection->new($conn, $responder), $env, \@handshake_results);
         });
     };
 }
@@ -112,8 +113,9 @@ Plack::App::WebSocket - WebSocket server as a PSGI application
                         ["Error: " . $env->{"plack.app.websocket.error"}]];
             },
             on_establish => sub {
-                my $conn = shift; ## Plack::App::WebSocket::Connection object
-                my $env = shift;  ## PSGI env
+                my $conn = shift;   ## Plack::App::WebSocket::Connection object
+                my $env = shift;    ## PSGI env
+                my $hs_res = shift; ## extra results from the handshake callback
                 $conn->on(
                     message => sub {
                         my ($conn, $msg) = @_;
@@ -172,10 +174,14 @@ WebSocket connection to a client.
 
 The code is called like
 
-    $code->($connection, $psgi_env)
+    $code->($connection, $psgi_env, \@handshake_results)
 
-where C<$connection> is a L<Plack::App::WebSocket::Connection> object
-and C<$psgi_env> is the PSGI environment object for the connection request.
+where C<$connection> is a L<Plack::App::WebSocket::Connection> object,
+C<$psgi_env> is the PSGI environment object for the connection request,
+and C<\@handshake_results> are extra results from the backend
+L<AnyEvent::WebSocket::Server> instance's handshake callback (which can
+be defined by passing a configured L<AnyEvent::WebSocket::Server> to
+the C<websocket_server> constructor parameter).
 You can use the C<$connection> to communicate with the client.
 
 Make sure you keep C<$connection> object as long as you need it.
